#
#  DeviceLabeler
#
#  $Id: plugin.py,v 1.0 2011/03/04 23:17:00 dre Exp $
#
#  Coded by dre (c) 2010
#  Coding idea and design by dre
#  Support: www.dreambox.tools
#
#  This plugin is licensed under the Creative Commons 
#  Attribution-NonCommercial-ShareAlike 3.0 Unported 
#  License. To view a copy of this license, visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
#
#  Alternatively, this plugin may be distributed and executed on hardware which
#  is licensed by Dream Multimedia GmbH.

#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially 
#  distributed other than under the conditions noted above.
#

from Components.config import config, ConfigText, NoSave, getConfigListEntry
from Components.ActionMap import *
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.MultiContent import MultiContentEntryText
from Components.Pixmap import Pixmap
from Plugins.Plugin import PluginDescriptor
from Screens.HelpMenu import HelpableScreen
from Screens.InputBox import InputBox
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Tools.BoundFunction import boundFunction
from enigma import RT_HALIGN_RIGHT, gFont, eConsoleAppContainer
import os
import re
import json

#new
from Components.MultiContent import MultiContentEntryText
from Components.Sources.List import List

def main(session,**kwargs):
	session.open(DeviceLabelerScreen)
	
class DeviceLabelerScreen(Screen, HelpableScreen):
	skin = """
		<screen position="center,center" size="1600,700" title="Device Labeler" >
			<widget name="deviceListText" position="20,20" size="200,40" font="Regular;25" foregroundColor="#f0f0f0" transparent="1" halign="center" valign="center" zPosition="5" />
			<widget source="deviceList" render="Listbox" position="0,100" size="800,330" scrollbarMode="showNever">
				<convert type="TemplatedMultiContent">
					{"template":
						[
							MultiContentEntryText(pos=(420,15), size=(360,30), font=0, text=0),
							MultiContentEntryText(pos=(420,45), size=(360,30), font=0, text=1),
							MultiContentEntryText(pos=(420,75), size=(360,30), font=0, text=2),
							MultiContentEntryText(pos=(420,105), size=(360,30), font=0, text=4),
							MultiContentEntryText(pos=(420,135), size=(360,30), font=0, text=5),
							MultiContentEntryText(pos=(420,165), size=(360,30), font=0, text=6),
							MultiContentEntryText(pos=(420,195), size=(360,30), font=0, text=7),
							MultiContentEntryText(pos=(420,225), size=(360,30), font=0, text=8),
							MultiContentEntryText(pos=(420,255), size=(360,30), font=0, text=9),
							MultiContentEntryText(pos=(420,285), size=(360,30), font=0, text=3),
							MultiContentEntryText(pos=(20,15), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Device:")),
							MultiContentEntryText(pos=(20,45), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("File system type:")),
							MultiContentEntryText(pos=(20,75), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Mount Point:")),	
							MultiContentEntryText(pos=(20,105), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Vendor:")),								
							MultiContentEntryText(pos=(20,135), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Model:")),
							MultiContentEntryText(pos=(20,165), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Size:")),		
							MultiContentEntryText(pos=(20,195), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Read only:")),
							MultiContentEntryText(pos=(20,225), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Removable:")),
							MultiContentEntryText(pos=(20,255), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Hotplugable:")),								
							MultiContentEntryText(pos=(20,285), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Current label:")),
					],
					"fonts": [gFont("Regular",25)],
					"itemHeight": 330
					}
				</convert>
			</widget>
			<widget name="partitionListText" position="820,20" size="200,40" font="Regular;25" foregroundColor="#f0f0f0" transparent="1" halign="center" valign="center" zPosition="5" />
			<widget source="partitionList" render="Listbox" position="800,100" size="600,540" scrollbarMode="showNever">
				<convert type="TemplatedMultiContent">
					{"template":[
									MultiContentEntryText(pos=(420,15), size=(360,30), font=0, text=0),
									MultiContentEntryText(pos=(420,45), size=(360,30), font=0, text=1),
									MultiContentEntryText(pos=(420,75), size=(360,30), font=0, text=2),
									MultiContentEntryText(pos=(420,105), size=(360,30), font=0, text=4),
									MultiContentEntryText(pos=(420,135), size=(360,30), font=0, text=3),
									MultiContentEntryText(pos=(20,15), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Partition:")),
									MultiContentEntryText(pos=(20,45), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("File system type:")),
									MultiContentEntryText(pos=(20,75), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Mount Point:")),
									MultiContentEntryText(pos=(20,105), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Size:")),
									MultiContentEntryText(pos=(20,135), size=(360,30), font=0, flags=RT_HALIGN_RIGHT, text=_("Current label:")),
					],
					"fonts": [gFont("Regular",25)],
					"itemHeight": 180
					}
				</convert>
			</widget>
			
			<widget name="buttonGreenText" position="20,650" size="200,40" font="Regular;25" foregroundColor="#f0f0f0" transparent="1" halign="center" valign="center" zPosition="5" />
			<ePixmap pixmap="/usr/share/enigma2/skin_default/buttons/green.png" position="20,650" size="200,40" />

			<widget name="buttonRedText" position="250,650" size="200,40" font="Regular;25" foregroundColor="#f0f0f0" transparent="1" halign="center" valign="center" zPosition="5" />
			<ePixmap pixmap="/usr/share/enigma2/skin_default/buttons/red.png" position="250,650" size="200,40" />

			<widget name="toggleText" position="630,650" size="200,30" font="Regular;25" foregroundColor="#f0f0f0" transparent="1"/>
			<ePixmap pixmap="/usr/share/enigma2/skin_default/icons/left.png" position="490,655" size="60,30" />
			<ePixmap pixmap="/usr/share/enigma2/skin_default/icons/right.png" position="560,655" size="60,30" />
		</screen>"""
		
	def __init__(self, session, args = 0):
		self.skin = DeviceLabelerScreen.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)

		self.deviceList = []
		self.currentList = "deviceList"
		self["deviceList"] = List(self.deviceList)
		self["deviceList"].setBuildFunc(self.buildDeviceEntry)
		self["partitionList"] = List([])
		self["partitionList"].setBuildFunc(self.buildPartitionEntry)
		self["partitionList"].setSelectionEnabled(False)
		
		self["buttonGreenText"] = Label(_("Change label"))
		self["buttonRedText"] = Label(_("Close"))
		
		self["deviceListText"] = Label(_("Devices"))
		self["partitionListText"] = Label(_("Partitions"))
	

		self["toggleText"] = Label(_("Toggle lists"))
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"cancel":	(self.close, _("Close plugin")),
		}, -1)
		
		
		self["ChannelSelectBaseActions"] = HelpableActionMap(self, "ChannelSelectBaseActions", 
		{
			"selectServicePageUp":		(boundFunction(self.moveSelection, "pageUp"), _("Page up")),
			"selectServicePageDown":	(boundFunction(self.moveSelection, "pageDown"), _("Page down")),
			"selectServiceUp":			(boundFunction(self.moveSelection, "moveUp"), _("Move up")),
			"selectServiceDown":		(boundFunction(self.moveSelection, "moveDown"), _("Move down")),	
			"prevMarker":				(self.toggleLists, _("Change active list")),
			"nextMarker":				(self.toggleLists, _("Change active list")),
			"showSatellites":			(self.changeLabel, _("Change label")),
			"showAllServices":			(self.close, _("Close plugin")),
	}, -5)

		self["deviceList"].onSelectionChanged.append(self.selectionChanged)

		self.container = eConsoleAppContainer()
		self.appClosed_conn = self.container.appClosed.connect(self.runFinished)
		self.dataAvail_conn = self.container.dataAvail.connect(self.dataAvail)

		self.cmdlist = ["lsblk -JO", "ls"]

		self.initCommands(0)
	
	def initCommands(self, index, cmd=None):
		self.run = index
		self.remainingData = ""
		self.allData = ""

		if cmd is not None:
			self.cmdlist[1] = cmd
		
		self.container.execute(self.cmdlist[self.run])
	
	def dataAvail(self, outputString):
		self.allData += outputString

	def runFinished(self, retval):
		if retval == 0:
			# cmd is lsblk -JO
			if self.run == 0:
				deviceData = json.loads(self.allData)
				self.deviceList = []
				deviceDict = {}
				self.partitionDict = {}
				for blockDevice in deviceData.get('blockdevices', []):
					partitionList = []
					
					bdName = blockDevice.get('name', '').encode('utf-8', 'ignore')
					majorminor = blockDevice.get('maj:min', '')
					if majorminor is not None:
						majorminorSplit = majorminor.split(':')
						if majorminorSplit[0] in ("1", "7", "9", "31"):
							print "[DeviceLabeler] - Device ignored because major %s is in blacklist" %(majorminorSplit[0])
							continue
						elif majorminorSplit[0] == "179":
							if bdName.endswith('rpmb') or bdName.startswith('mmcblk0boot'):
								print "[DeviceLabeler] - Device ignored because %s is a system device" %(bdName)
								continue


					fsType = blockDevice.get('fstype', '')
					if fsType is not None:
						fsType = fsType.encode('utf-8', 'ignore')

					mountPoint = blockDevice.get('mountpoint', '')
					if mountPoint is not None:
						mountPoint = mountPoint.encode('utf-8','ignore')

					label = blockDevice.get('label', '')
					if label is not None:
						label = label.encode('utf-8','ignore')

					vendor = blockDevice.get('vendor', '')
					if vendor is not None:
						vendor = vendor.encode('utf-8', 'ignore')
					else:
						vendor = _("Unknown")
						
					model = blockDevice.get('model', '')
					if model is not None:
						model = model.encode('utf-8', 'ignore')
					else:
						model = _("Unknown")

					size = blockDevice.get('size', '')
					if size is not None:
						size = size.encode('utf-8', 'ignore')
						
					ro = blockDevice.get('ro').encode('utf-8', 'ignore')
					rm = blockDevice.get('rm').encode('utf-8', 'ignore')
					hotplug = blockDevice.get('hotplug').encode('utf-8', 'ignore')

					self.deviceList.append((bdName, fsType, mountPoint, label, vendor, model, size, ro, rm, hotplug))
					deviceDict[bdName] = self.deviceList
					for child in blockDevice.get('children', []):
						partName = child.get('name', '').encode('utf-8', 'ignore')
						partFsType = child.get('fstype')
						if partFsType is not None:
							partFsType = partFsType.encode('utf-8', 'ignore')
						partMountPoint = child.get('mountpoint')
						if partMountPoint is not None:
							partMountPoint = partMountPoint.encode('utf-8', 'ignore')
						partLabel = child.get('label')
						if partLabel is not None:
							partLabel = partLabel.encode('utf-8', 'ignore')
						partPartLabel = child.get('partlabel')
						if partPartLabel is not None:
							partPartLabel = partPartLabel.encode('utf-8', 'ignore')
						partSize = child.get('size').encode('utf-8', 'ignore')
						
						partitionList.append((partName, partFsType, partMountPoint, partLabel, partPartLabel, partSize ))
					if len(partitionList):
						self.partitionDict[bdName] = partitionList
				
				self["deviceList"].setList(self.deviceList)
			# cmd is tune2fs -L <newLabel> /dev/<device>
			elif self.run == 1:
				self.initCommands(0)
				

	def buildDeviceEntry(self, bdName, fsType, mountPoint, label, vendor, model, size, ro, rm, hotplug):
		rotext = _("No")
		if ro == "1":
			rotext = _("Yes")
		rmtext = _("No")
		if rm == "1":
			rmtext = _("Yes")
		hotplugtext = _("No")
		if hotplug == "1":
			hotplugtext = _("Yes")
		
		print bdName, fsType, mountPoint, label, ro, rmtext,  hotplugtext, model, size, vendor
		return [ bdName, fsType, mountPoint, label, vendor, model, size, rotext, rmtext, hotplugtext]
		
		
	def buildPartitionEntry(self, partName, partFsType, partMountPoint, partLabel, partPartLabel, partSize):
	
		print "partEntry", partName, partFsType, partMountPoint, partLabel, partPartLabel, partSize
		partLabelText = partLabel
		if partLabel is None:
			partLabelText = partPartLabel
			
		return [partName, partFsType, partMountPoint, partLabelText, partSize]

	def selectionChanged(self):
		cur = self["deviceList"].getCurrent()
		
		if cur:
			partitionList = self.partitionDict.get(cur[0], [])
			self["partitionList"].setList(partitionList)
	
	def toggleLists(self):
		if self.currentList == "deviceList":
			self.currentList = "partitionList"
			self[self.currentList].setSelectionEnabled(True)
		else:
			self[self.currentList].setSelectionEnabled(False)
			self.currentList = "deviceList"
			self[self.currentList].setSelectionEnabled(True)
	
	def moveSelection(self, dir):
		self[self.currentList].moveSelection(dir)
		
	def pageUp(self):
		self[self.currentList].pageUp()
		
	def pageDown(self):
		self[self.currentList].pageDown()
		
	def left(self):
		self[self.currentList].moveSelection("moveUp")

	def right(self):
		self[self.currentList].moveSelection("moveDown")
		
	def changeLabel(self):
		cur = self[self.currentList].getCurrent()
		if self.currentList == "partitionList":
			if cur:
				if cur[3] in ('kernel', 'dreambox-rootfs', 'dreambox-data'):
					self.session.open(MessageBox, _("Label %s cannot be changed" %(cur[3])), MessageBox.TYPE_INFO, timeout=5)
				elif not cur[1] in ('ext2', 'ext3', 'ext4'):
					self.session.open(MessageBox, _("A label can only be changed for %s formatted in ext2, ext3 or ext4" %("partitions")), MessageBox.TYPE_INFO, timeout=5)
				else:
					self.session.openWithCallback(boundFunction(self.setNewLabel, cur[0]), InputBox, windowTitle = _("Set new label for /dev/%s" %(cur[0])), title = _("Please enter new label"), text="")
		else:
			if cur:
				if self.partitionDict.get(cur[0]) is not None:
					self.session.open(MessageBox, _("A device can only have a label if no partitions exists"), MessageBox.TYPE_INFO, timeout=5)
				elif not cur[1] in ('ext2', 'ext3', 'ext4'):
					self.session.open(MessageBox, _("A label can only be changed for %s formatted in ext2, ext3 or ext4" %("devices")), MessageBox.TYPE_INFO, timeout=5)
				else:
					self.session.openWithCallback(boundFunction(self.setNewLabel, cur[0]), InputBox, windowTitle = _("Set new label for /dev/%s" %(cur[0])), title = _("Please enter new label"), text="")

	def setNewLabel(self, device, newLabel):
		if newLabel:
			self.initCommands(1, "tune2fs -L %s /dev/%s" %(newLabel, device))

def Plugins(**kwargs):
	return [PluginDescriptor(name="Device Labeler", description=_("Plugin to change the label of a device"), where = [PluginDescriptor.WHERE_PLUGINMENU], icon="DeviceLabeler.png", fnc=main)]
